@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mplayer.c,v 1.6 2001/10/25 04:56:26 slouken Exp $
+  $Id: mplayer.c,v 1.7 2001/12/19 17:11:40 slouken Exp $
 
   The Protracker Player Driver
 
@@ -1806,6 +1806,9 @@ static void pt_playeffects(void)
 			case UNI_MEDEFFECTF3:
 				DoEEffects(0x90|(pf->sngspd/3));
 				break;
+			case UNI_XMEFFECTZ:
+				Player_SetSynchroValue(UniGetByte());
+				break;
 			default:
 				a->sliding=oldsliding;
 				UniSkipOpcode(c);
@@ -2897,4 +2900,16 @@ void Player_SetTempo(UWORD tempo)
 	MUTEX_UNLOCK(vars);
 }
 
+static int _pl_synchro_value;
+void Player_SetSynchroValue(int i)
+{
+	_pl_synchro_value = i;
+}
+
+int Player_GetSynchroValue(void)
+{
+	return _pl_synchro_value;
+}
+	
+
 /* ex:set ts=4: */
