@@ -1,6 +1,6 @@
 /*
-    MIXERLIB:  An audio mixer library based on the SDL library
-    Copyright (C) 1997-1999  Sam Lantinga
+    SDL_mixer:  An audio mixer library based on the SDL library
+    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -17,12 +17,10 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
     Sam Lantinga
-    5635-34 Springhouse Dr.
-    Pleasanton, CA 94588 (USA)
-    slouken@devolution.com
+    slouken@libsdl.org
 */
 
-/* $Id: mixer.c,v 1.22 2001/03/04 22:48:51 hercules Exp $ */
+/* $Id: mixer.c,v 1.39 2002/05/19 20:50:35 slouken Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,11 +31,25 @@
 #include "SDL_timer.h"
 
 #include "SDL_mixer.h"
+#include "load_aiff.h"
+#include "load_voc.h"
+
+/* Magic numbers for various audio file formats */
+#define RIFF		0x46464952		/* "RIFF" */
+#define WAVE		0x45564157		/* "WAVE" */
+#define FORM		0x4d524f46		/* "FORM" */
 
 static int audio_opened = 0;
-
 static SDL_AudioSpec mixer;
-static SDL_mutex *mixer_lock;
+
+typedef struct _Mix_effectinfo
+{
+	Mix_EffectFunc_t callback;
+	Mix_EffectDone_t done_callback;
+	void *udata;
+	struct _Mix_effectinfo *next;
+} effect_info;
+
 static struct _Mix_Channel {
 	Mix_Chunk *chunk;
 	int playing;
@@ -52,7 +64,11 @@ static struct _Mix_Channel {
 	int fade_volume;
 	Uint32 fade_length;
 	Uint32 ticks_fade;
+	effect_info *effects;
 } *mix_channel = NULL;
+
+static effect_info *posteffects = NULL;
+
 static int num_channels;
 static int reserved_channels = 0;
 
@@ -61,6 +77,9 @@ static int reserved_channels = 0;
 static void (*mix_postmix)(void *udata, Uint8 *stream, int len) = NULL;
 static void *mix_postmix_data = NULL;
 
+/* rcg07062001 callback to alert when channels are done playing. */
+static void (*channel_done_callback)(int channel) = NULL;
+
 /* Music function declarations */
 extern int open_music(SDL_AudioSpec *mixer);
 extern void close_music(void);
@@ -71,9 +90,68 @@ extern void music_mixer(void *udata, Uint8 *stream, int len);
 static void (*mix_music)(void *udata, Uint8 *stream, int len) = music_mixer;
 static void *music_data = NULL;
 
+
+/* rcg06192001 get linked library's version. */
+const SDL_version *Mix_Linked_Version(void)
+{
+	static SDL_version linked_mixver;
+	MIX_VERSION(&linked_mixver);
+	return(&linked_mixver);
+}
+
+static int _Mix_remove_all_effects(int channel, effect_info **e);
+
+/*
+ * rcg06122001 Cleanup effect callbacks.
+ *  MAKE SURE SDL_LockAudio() is called before this (or you're in the
+ *   audio callback).
+ */
+static void _Mix_channel_done_playing(int channel)
+{
+	if (channel_done_callback) {
+	    channel_done_callback(channel);
+	}
+
+	/*
+	 * Call internal function directly, to avoid locking audio from
+	 *   inside audio callback.
+	 */
+	_Mix_remove_all_effects(channel, &mix_channel[channel].effects);
+}
+
+
+static void *Mix_DoEffects(int chan, void *snd, int len)
+{
+	int posteffect = (chan == MIX_CHANNEL_POST);
+	effect_info *e = ((posteffect) ? posteffects : mix_channel[chan].effects);
+	void *buf = snd;
+
+	if (e != NULL) {    /* are there any registered effects? */
+		/* if this is the postmix, we can just overwrite the original. */
+		if (!posteffect) {
+			buf = malloc(len);
+			if (buf == NULL) {
+				return(snd);
+			}
+		    memcpy(buf, snd, len);
+		}
+
+		for (; e != NULL; e = e->next) {
+			if (e->callback != NULL) {
+				e->callback(chan, buf, len, e->udata);
+			}
+		}
+	}
+
+	/* be sure to free() the return value if != snd ... */
+	return(buf);
+}
+
+
 /* Mixing function */
 static void mix_channels(void *udata, Uint8 *stream, int len)
 {
+	Uint8 *mix_input;
 	int i, mixable, volume;
 	Uint32 sdl_ticks;
 
@@ -82,8 +160,7 @@ static void mix_channels(void *udata, Uint8 *stream, int len)
 		mix_music(music_data, stream, len);
 	}
 
-	/* Grab the channels we need to mix */
-	SDL_mutexP(mixer_lock);
+	/* Mix any playing channels... */
 	sdl_ticks = SDL_GetTicks();
 	for ( i=0; i<num_channels; ++i ) {
 		if( ! mix_channel[i].paused ) {
@@ -116,7 +193,12 @@ static void mix_channels(void *udata, Uint8 *stream, int len)
 				if ( mixable > len ) {
 					mixable = len;
 				}
-				SDL_MixAudio(stream,mix_channel[i].samples,mixable,volume);
+
+				mix_input = Mix_DoEffects(i, mix_channel[i].samples, mixable);
+				SDL_MixAudio(stream,mix_input,mixable,volume);
+				if (mix_input != mix_channel[i].samples)
+					free(mix_input);
+
 				mix_channel[i].samples += mixable;
 				mix_channel[i].playing -= mixable;
 				/* If looping the sample and we are at its end, make sure
@@ -127,7 +209,12 @@ static void mix_channels(void *udata, Uint8 *stream, int len)
 				    	if (remaining > alen) {
 						remaining = alen;
 				    	}
-					SDL_MixAudio(stream+mixable, mix_channel[i].chunk->abuf, remaining, volume);
+
+					mix_input = Mix_DoEffects(i, mix_channel[i].chunk->abuf, remaining);
+					SDL_MixAudio(stream+mixable, mix_input, remaining, volume);
+					if (mix_input != mix_channel[i].chunk->abuf)
+						free(mix_input);
+
 					--mix_channel[i].looping;
 					mix_channel[i].samples = mix_channel[i].chunk->abuf + remaining;
 					mix_channel[i].playing = mix_channel[i].chunk->alen - remaining;
@@ -139,13 +226,21 @@ static void mix_channels(void *udata, Uint8 *stream, int len)
 						mix_channel[i].playing = mix_channel[i].chunk->alen;
 					}
 				}
+
+				/* rcg06072001 Alert app if channel is done playing. */
+				if (!mix_channel[i].playing) {
+					_Mix_channel_done_playing(i);
+				}
 			}
 		}
 	}
+
+	/* rcg06122001 run posteffects... */
+	Mix_DoEffects(MIX_CHANNEL_POST, stream, len);
+
 	if ( mix_postmix ) {
 		mix_postmix(mix_postmix_data, stream, len);
 	}
-	SDL_mutexV(mixer_lock);
 }
 
 static void PrintFormat(char *title, SDL_AudioSpec *fmt)
@@ -155,6 +250,9 @@ static void PrintFormat(char *title, SDL_AudioSpec *fmt)
 			(fmt->channels > 1) ? "stereo" : "mono", fmt->freq);
 }
 
+
+void _Mix_InitEffects(void);
+
 /* Open the mixer with a certain desired audio format */
 int Mix_OpenAudio(int frequency, Uint16 format, int nchannels, int chunksize)
 {
@@ -183,20 +281,9 @@ int Mix_OpenAudio(int frequency, Uint16 format, int nchannels, int chunksize)
 	PrintFormat("Audio device", &mixer);
 #endif
 
-	/* Create the channel lock mutex */
-	mixer_lock = SDL_CreateMutex();
-#ifndef macintosh /* Hmm.. what implications does this have? */
-	if ( mixer_lock == NULL ) {
-		SDL_CloseAudio();
-		SDL_SetError("Unable to create mixer lock");
-		return(-1);
-	}
-#endif
-
 	/* Initialize the music players */
 	if ( open_music(&mixer) < 0 ) {
 		SDL_CloseAudio();
-		SDL_DestroyMutex(mixer_lock);
 		return(-1);
 	}
 
@@ -211,8 +298,12 @@ int Mix_OpenAudio(int frequency, Uint16 format, int nchannels, int chunksize)
 		mix_channel[i].volume = SDL_MIX_MAXVOLUME;
 		mix_channel[i].tag = -1;
 		mix_channel[i].expire = 0;
+		mix_channel[i].effects = NULL;
 	}
 	Mix_VolumeMusic(SDL_MIX_MAXVOLUME);
+
+	_Mix_InitEffects();
+
 	audio_opened = 1;
 	SDL_PauseAudio(0);
 	return(0);
@@ -234,7 +325,7 @@ int Mix_AllocateChannels(int numchans)
 			Mix_HaltChannel(i);
 		}
 	}
-	SDL_mutexP(mixer_lock);
+	SDL_LockAudio();
 	mix_channel = (struct _Mix_Channel *) realloc(mix_channel, numchans * sizeof(struct _Mix_Channel));
 	if ( numchans > num_channels ) {
 		/* Initialize the new channels */
@@ -246,10 +337,11 @@ int Mix_AllocateChannels(int numchans)
 			mix_channel[i].volume = SDL_MIX_MAXVOLUME;
 			mix_channel[i].tag = -1;
 			mix_channel[i].expire = 0;
+			mix_channel[i].effects = NULL;
 		}
 	}
 	num_channels = numchans;
-	SDL_mutexV(mixer_lock);
+	SDL_UnlockAudio();
 	return(num_channels);
 }
 
@@ -270,18 +362,31 @@ int Mix_QuerySpec(int *frequency, Uint16 *format, int *channels)
 	return(audio_opened);
 }
 
+
+/*
+ * !!! FIXME: Ideally, we want a Mix_LoadSample_RW(), which will handle the
+ *             generic setup, then call the correct file format loader.
+ */
+
 /* Load a wave file */
 Mix_Chunk *Mix_LoadWAV_RW(SDL_RWops *src, int freesrc)
 {
+	Uint32 magic;
 	Mix_Chunk *chunk;
-	SDL_AudioSpec wavespec;
+	SDL_AudioSpec wavespec, *loaded;
 	SDL_AudioCVT wavecvt;
 	int samplesize;
 
+	/* rcg06012001 Make sure src is valid */
+	if ( ! src ) {
+		SDL_SetError("Mix_LoadWAV_RW with NULL src");
+		return(NULL);
+	}
+
 	/* Make sure audio has been opened */
 	if ( ! audio_opened ) {
 		SDL_SetError("Audio device hasn't been opened");
-		if ( freesrc ) {
+		if ( freesrc && src ) {
 			SDL_RWclose(src);
 		}
 		return(NULL);
@@ -297,12 +402,31 @@ Mix_Chunk *Mix_LoadWAV_RW(SDL_RWops *src, int freesrc)
 		return(NULL);
 	}
 
-	/* Load the WAV file into the chunk */
-	if ( SDL_LoadWAV_RW(src, freesrc,
-		&wavespec, (Uint8 **)&chunk->abuf, &chunk->alen) == NULL ) {
+	/* Find out what kind of audio file this is */
+	magic = SDL_ReadLE32(src);
+	/* Seek backwards for compatibility with older loaders */
+	SDL_RWseek(src, -(int)sizeof(Uint32), SEEK_CUR);
+
+	switch (magic) {
+		case WAVE:
+		case RIFF:
+			loaded = SDL_LoadWAV_RW(src, freesrc, &wavespec,
+					(Uint8 **)&chunk->abuf, &chunk->alen);
+			break;
+		case FORM:
+			loaded = Mix_LoadAIFF_RW(src, freesrc, &wavespec,
+					(Uint8 **)&chunk->abuf, &chunk->alen);
+			break;
+		default:
+			loaded = Mix_LoadVOC_RW(src, freesrc, &wavespec,
+					(Uint8 **)&chunk->abuf, &chunk->alen);
+			break;
+	}
+	if ( !loaded ) {
 		free(chunk);
 		return(NULL);
 	}
+
 #if 0
 	PrintFormat("Audio device", &mixer);
 	PrintFormat("-- Wave file", &wavespec);
@@ -376,6 +500,33 @@ Mix_Chunk *Mix_QuickLoad_WAV(Uint8 *mem)
 	return(chunk);
 }
 
+/* Load raw audio data of the mixer format from a memory buffer */
+Mix_Chunk *Mix_QuickLoad_RAW(Uint8 *mem, Uint32 len)
+{
+	Mix_Chunk *chunk;
+
+	/* Make sure audio has been opened */
+	if ( ! audio_opened ) {
+		SDL_SetError("Audio device hasn't been opened");
+		return(NULL);
+	}
+
+	/* Allocate the chunk memory */
+	chunk = (Mix_Chunk *)malloc(sizeof(Mix_Chunk));
+	if ( chunk == NULL ) {
+		SDL_SetError("Out of memory");
+		return(NULL);
+	}
+
+	/* Essentially just point at the audio data (no error checking - fast) */
+	chunk->allocated = 0;
+	chunk->alen = len;
+	chunk->abuf = mem;
+	chunk->volume = MIX_MAX_VOLUME;
+
+	return(chunk);
+}
+
 /* Free an audio chunk previously loaded */
 void Mix_FreeChunk(Mix_Chunk *chunk)
 {
@@ -384,16 +535,15 @@ void Mix_FreeChunk(Mix_Chunk *chunk)
 	/* Caution -- if the chunk is playing, the mixer will crash */
 	if ( chunk ) {
 		/* Guarantee that this chunk isn't playing */
+		SDL_LockAudio();
 		if ( mix_channel ) {
-			SDL_mutexP(mixer_lock);
 			for ( i=0; i<num_channels; ++i ) {
 				if ( chunk == mix_channel[i].chunk ) {
 					mix_channel[i].playing = 0;
 				}
 			}
-			SDL_mutexV(mixer_lock);
 		}
-
+		SDL_UnlockAudio();
 		/* Actually free the chunk */
 		if ( chunk->allocated ) {
 			free(chunk->abuf);
@@ -437,6 +587,14 @@ void *Mix_GetMusicHookData(void)
 	return(music_data);
 }
 
+void Mix_ChannelFinished(void (*channel_finished)(int channel))
+{
+	SDL_LockAudio();
+	channel_done_callback = channel_finished;
+	SDL_UnlockAudio();
+}
+
+
 /* Reserve the first channels (0 -> n-1) for the application, i.e. don't allocate
    them dynamically to the next sample if requested with a -1 value below.
    Returns the number of reserved channels.
@@ -461,11 +619,12 @@ int Mix_PlayChannelTimed(int which, Mix_Chunk *chunk, int loops, int ticks)
 
 	/* Don't play null pointers :-) */
 	if ( chunk == NULL ) {
+		Mix_SetError("Tried to play a NULL chunk");
 		return(-1);
 	}
 
 	/* Lock the mixer while modifying the playing channels */
-	SDL_mutexP(mixer_lock);
+	SDL_LockAudio();
 	{
 		/* If which is -1, play on the first free channel */
 		if ( which == -1 ) {
@@ -474,6 +633,7 @@ int Mix_PlayChannelTimed(int which, Mix_Chunk *chunk, int loops, int ticks)
 					break;
 			}
 			if ( i == num_channels ) {
+				Mix_SetError("No free channels available");
 				which = -1;
 			} else {
 				which = i;
@@ -483,6 +643,8 @@ int Mix_PlayChannelTimed(int which, Mix_Chunk *chunk, int loops, int ticks)
 		/* Queue up the audio data for this channel */
 		if ( which >= 0 ) {
 			Uint32 sdl_ticks = SDL_GetTicks();
+			if (Mix_Playing(which))
+				_Mix_channel_done_playing(which);
 			mix_channel[which].samples = chunk->abuf;
 			mix_channel[which].playing = chunk->alen;
 			mix_channel[which].looping = loops;
@@ -493,7 +655,7 @@ int Mix_PlayChannelTimed(int which, Mix_Chunk *chunk, int loops, int ticks)
 			mix_channel[which].expire = (ticks>0) ? (sdl_ticks + ticks) : 0;
 		}
 	}
-	SDL_mutexV(mixer_lock);
+	SDL_UnlockAudio();
 
 	/* Return the channel on which the sound is being played */
 	return(which);
@@ -510,9 +672,9 @@ int Mix_ExpireChannel(int which, int ticks)
 			status += Mix_ExpireChannel(i, ticks);
 		}
 	} else if ( which < num_channels ) {
-		SDL_mutexP(mixer_lock);
+		SDL_LockAudio();
 		mix_channel[which].expire = (ticks>0) ? (SDL_GetTicks() + ticks) : 0;
-		SDL_mutexV(mixer_lock);
+		SDL_UnlockAudio();
 		++ status;
 	}
 	return(status);
@@ -529,7 +691,7 @@ int Mix_FadeInChannelTimed(int which, Mix_Chunk *chunk, int loops, int ms, int t
 	}
 
 	/* Lock the mixer while modifying the playing channels */
-	SDL_mutexP(mixer_lock);
+	SDL_LockAudio();
 	{
 		/* If which is -1, play on the first free channel */
 		if ( which == -1 ) {
@@ -547,6 +709,8 @@ int Mix_FadeInChannelTimed(int which, Mix_Chunk *chunk, int loops, int ms, int t
 		/* Queue up the audio data for this channel */
 		if ( which >= 0 ) {
 			Uint32 sdl_ticks = SDL_GetTicks();
+			if (Mix_Playing(which))
+				_Mix_channel_done_playing(which);
 			mix_channel[which].samples = chunk->abuf;
 			mix_channel[which].playing = chunk->alen;
 			mix_channel[which].looping = loops;
@@ -560,7 +724,7 @@ int Mix_FadeInChannelTimed(int which, Mix_Chunk *chunk, int loops, int ms, int t
 			mix_channel[which].expire = (ticks > 0) ? (sdl_ticks+ticks) : 0;
 		}
 	}
-	SDL_mutexV(mixer_lock);
+	SDL_UnlockAudio();
 
 	/* Return the channel on which the sound is being played */
 	return(which);
@@ -580,13 +744,12 @@ int Mix_Volume(int which, int volume)
 		prev_volume /= num_channels;
 	} else {
 		prev_volume = mix_channel[which].volume;
-		if ( volume < 0 ) {
-			volume = 0;
+		if ( volume >= 0 ) {
+			if ( volume > SDL_MIX_MAXVOLUME ) {
+				volume = SDL_MIX_MAXVOLUME;
+			}
+			mix_channel[which].volume = volume;
 		}
-		if ( volume > SDL_MIX_MAXVOLUME ) {
-			volume = SDL_MIX_MAXVOLUME;
-		}
-		mix_channel[which].volume = volume;
 	}
 	return(prev_volume);
 }
@@ -615,13 +778,16 @@ int Mix_HaltChannel(int which)
 			Mix_HaltChannel(i);
 		}
 	} else {
-		SDL_mutexP(mixer_lock);
+		SDL_LockAudio();
+		if (mix_channel[which].playing) {
+			_Mix_channel_done_playing(which);
 		mix_channel[which].playing = 0;
+		}
 		mix_channel[which].expire = 0;
 		if(mix_channel[which].fading != MIX_NO_FADING) /* Restore volume */
 			mix_channel[which].volume = mix_channel[which].fade_volume;
 		mix_channel[which].fading = MIX_NO_FADING;
-		SDL_mutexV(mixer_lock);
+		SDL_UnlockAudio();
 	}
 	return(0);
 }
@@ -645,24 +811,27 @@ int Mix_FadeOutChannel(int which, int ms)
 	int status;
 
 	status = 0;
-	if ( which == -1 ) {
-		int i;
+	if ( audio_opened ) {
+		if ( which == -1 ) {
+			int i;
 
-		for ( i=0; i<num_channels; ++i ) {
-			status += Mix_FadeOutChannel(i,ms);
-		}
-	} else {
-		SDL_mutexP(mixer_lock);
-		if ( mix_channel[which].playing && mix_channel[which].volume>0 &&
-			 mix_channel[which].fading!=MIX_FADING_OUT ) {
+			for ( i=0; i<num_channels; ++i ) {
+				status += Mix_FadeOutChannel(i, ms);
+			}
+		} else {
+			SDL_LockAudio();
+			if ( mix_channel[which].playing && 
+			    (mix_channel[which].volume > 0) &&
+			    (mix_channel[which].fading != MIX_FADING_OUT) ) {
 
-			mix_channel[which].fading = MIX_FADING_OUT;
-			mix_channel[which].fade_volume = mix_channel[which].volume;
-			mix_channel[which].fade_length = ms;
-			mix_channel[which].ticks_fade = SDL_GetTicks();
-			++ status;
+				mix_channel[which].fading = MIX_FADING_OUT;
+				mix_channel[which].fade_volume = mix_channel[which].volume;
+				mix_channel[which].fade_length = ms;
+				mix_channel[which].ticks_fade = SDL_GetTicks();
+				++status;
+			}
+			SDL_UnlockAudio();
 		}
-		SDL_mutexV(mixer_lock);
 	}
 	return(status);
 }
@@ -686,7 +855,7 @@ Mix_Fading Mix_FadingChannel(int which)
 }
 
 /* Check the status of a specific channel.
-   If the specified mix_channel is -1, check all mix_channels.
+   If the specified mix_channel is -1, check all mix channels.
 */
 int Mix_Playing(int which)
 {
@@ -697,27 +866,48 @@ int Mix_Playing(int which)
 		int i;
 
 		for ( i=0; i<num_channels; ++i ) {
-			if ( mix_channel[i].playing > 0 ) {
+			if ((mix_channel[i].playing > 0) ||
+				(mix_channel[i].looping > 0))
+			{
 				++status;
 			}
 		}
 	} else {
-		if ( mix_channel[which].playing > 0 ) {
+		if ((mix_channel[which].playing > 0) ||
+			(mix_channel[which].looping > 0))
+		{
 			++status;
 		}
 	}
 	return(status);
 }
 
+/* rcg06072001 Get the chunk associated with a channel. */
+Mix_Chunk *Mix_GetChunk(int channel)
+{
+	Mix_Chunk *retval = NULL;
+
+	if ((channel >= 0) && (channel < num_channels)) {
+		retval = mix_channel[channel].chunk;
+	}
+
+	return(retval);
+}
+
 /* Close the mixer, halting all playing audio */
 void Mix_CloseAudio(void)
 {
+	int i;
+
 	if ( audio_opened ) {
 		if ( audio_opened == 1 ) {
+			for (i = 0; i < num_channels; i++) {
+				Mix_UnregisterAllEffects(i);
+			}
+			Mix_UnregisterAllEffects(MIX_CHANNEL_POST);
 			close_music();
 			Mix_HaltChannel(-1);
 			SDL_CloseAudio();
-			SDL_DestroyMutex(mixer_lock);
 			free(mix_channel);
 			mix_channel = NULL;
 		}
@@ -748,10 +938,11 @@ void Mix_Pause(int which)
 void Mix_Resume(int which)
 {
 	Uint32 sdl_ticks = SDL_GetTicks();
+
+	SDL_LockAudio();
 	if ( which == -1 ) {
 		int i;
 
-		SDL_mutexP(mixer_lock);
 		for ( i=0; i<num_channels; ++i ) {
 			if ( mix_channel[i].playing > 0 ) {
 				if(mix_channel[i].expire > 0)
@@ -759,16 +950,14 @@ void Mix_Resume(int which)
 				mix_channel[i].paused = 0;
 			}
 		}
-		SDL_mutexV(mixer_lock);
 	} else {
-		SDL_mutexP(mixer_lock);
 		if ( mix_channel[which].playing > 0 ) {
 			if(mix_channel[which].expire > 0)
 				mix_channel[which].expire += sdl_ticks - mix_channel[which].paused;
 			mix_channel[which].paused = 0;
 		}
-		SDL_mutexV(mixer_lock);
 	}
+	SDL_UnlockAudio();
 }
 
 int Mix_Paused(int which)
@@ -795,9 +984,9 @@ int Mix_GroupChannel(int which, int tag)
 	if ( which < 0 || which > num_channels )
 		return(0);
 
-	SDL_mutexP(mixer_lock);
+	SDL_LockAudio();
 	mix_channel[which].tag = tag;
-	SDL_mutexV(mixer_lock);
+	SDL_UnlockAudio();
 	return(1);
 }
 
@@ -865,3 +1054,183 @@ int Mix_GroupNewer(int tag)
 	}
 	return(chan);
 }
+
+
+
+/*
+ * rcg06122001 The special effects exportable API.
+ *  Please see effect_*.c for internally-implemented effects, such
+ *  as Mix_SetPanning().
+ */
+
+/* MAKE SURE you hold the audio lock (SDL_LockAudio()) before calling this! */
+static int _Mix_register_effect(effect_info **e, Mix_EffectFunc_t f,
+				Mix_EffectDone_t d, void *arg)
+{
+	effect_info *new_e = malloc(sizeof (effect_info));
+
+	if (!e) {
+		Mix_SetError("Internal error");
+		return(0);
+	}
+
+	if (f == NULL) {
+		Mix_SetError("NULL effect callback");
+		return(0);
+	}
+
+	if (new_e == NULL) {
+		Mix_SetError("Out of memory");
+		return(0);
+	}
+
+	new_e->callback = f;
+	new_e->done_callback = d;
+	new_e->udata = arg;
+	new_e->next = NULL;
+
+	/* add new effect to end of linked list... */
+	if (*e == NULL) {
+		*e = new_e;
+	} else {
+		effect_info *cur = *e;
+		while (1) {
+			if (cur->next == NULL) {
+				cur->next = new_e;
+				break;
+			}
+			cur = cur->next;
+		}
+	}
+
+	return(1);
+}
+
+
+/* MAKE SURE you hold the audio lock (SDL_LockAudio()) before calling this! */
+static int _Mix_remove_effect(int channel, effect_info **e, Mix_EffectFunc_t f)
+{
+	effect_info *cur;
+	effect_info *prev = NULL;
+	effect_info *next = NULL;
+
+	if (!e) {
+		Mix_SetError("Internal error");
+		return(0);
+	}
+
+	for (cur = *e; cur != NULL; cur = cur->next) {
+		if (cur->callback == f) {
+			next = cur->next;
+			if (cur->done_callback != NULL) {
+				cur->done_callback(channel, cur->udata);
+			}
+			free(cur);
+
+			if (prev == NULL) {   /* removing first item of list? */
+				*e = next;
+			} else {
+				prev->next = next;
+			}
+			return(1);
+		}
+		prev = cur;
+	}
+
+	Mix_SetError("No such effect registered");
+	return(0);
+}
+
+
+/* MAKE SURE you hold the audio lock (SDL_LockAudio()) before calling this! */
+static int _Mix_remove_all_effects(int channel, effect_info **e)
+{
+	effect_info *cur;
+	effect_info *next;
+
+	if (!e) {
+		Mix_SetError("Internal error");
+		return(0);
+	}
+
+	for (cur = *e; cur != NULL; cur = next) {
+		next = cur->next;
+		if (cur->done_callback != NULL) {
+			cur->done_callback(channel, cur->udata);
+		}
+		free(cur);
+	}
+	*e = NULL;
+
+	return(1);
+}
+
+
+int Mix_RegisterEffect(int channel, Mix_EffectFunc_t f,
+			Mix_EffectDone_t d, void *arg)
+{
+	effect_info **e = NULL;
+	int retval;
+
+	if (channel == MIX_CHANNEL_POST) {
+		e = &posteffects;
+	} else {
+		if ((channel < 0) || (channel >= num_channels)) {
+			Mix_SetError("Invalid channel number");
+			return(0);
+		}
+		e = &mix_channel[channel].effects;
+	}
+
+	SDL_LockAudio();
+	retval = _Mix_register_effect(e, f, d, arg);
+	SDL_UnlockAudio();
+	return(retval);
+}
+
+
+int Mix_UnregisterEffect(int channel, Mix_EffectFunc_t f)
+{
+	effect_info **e = NULL;
+	int retval;
+
+	if (channel == MIX_CHANNEL_POST) {
+		e = &posteffects;
+	} else {
+		if ((channel < 0) || (channel >= num_channels)) {
+			Mix_SetError("Invalid channel number");
+			return(0);
+		}
+		e = &mix_channel[channel].effects;
+	}
+
+	SDL_LockAudio();
+	retval = _Mix_remove_effect(channel, e, f);
+	SDL_UnlockAudio();
+	return(retval);
+}
+
+
+int Mix_UnregisterAllEffects(int channel)
+{
+	effect_info **e = NULL;
+	int retval;
+
+	if (channel == MIX_CHANNEL_POST) {
+		e = &posteffects;
+	} else {
+		if ((channel < 0) || (channel >= num_channels)) {
+			Mix_SetError("Invalid channel number");
+			return(0);
+		}
+		e = &mix_channel[channel].effects;
+	}
+
+	SDL_LockAudio();
+	retval = _Mix_remove_all_effects(channel, e);
+	SDL_UnlockAudio();
+	return(retval);
+}
+
+/* end of mixer.c ... */
+
